﻿using Newtonsoft.Json;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading;
using System.Web;

namespace _12306Common
{
    public class PiaoTask
    {
        private ConcurrentQueue<string> ips;
        private Setting setting;
        private string secretStr;
        public string ipStr;

        public PiaoTask()
        {

        }

        public string Run(Setting setting)
        {
            this.setting = setting;
            this.ips = new ConcurrentQueue<string>(setting.Ips);
            //现在ips里有600个IP，大概10个线程有了，从ips队列里出列查询
            for (var i = 0; i < setting.Ips.Count / 50 + 1; i++)
            {
                var worker = new BackgroundWorker();
                worker.DoWork += worker_DoWork;
                worker.RunWorkerAsync();
            }

            while (true)
            {
                Thread.Sleep(300);

                if (!string.IsNullOrEmpty(secretStr) || this.ips.Count() == 0)
                    break;
            }

            return secretStr;
        }

        void worker_DoWork(object s, DoWorkEventArgs e)
        {
            while (true)
            {
                //所有线程看到其他线程找到了secretStr，退出
                if (!string.IsNullOrEmpty(secretStr))
                    break;

                var ip = string.Empty;
                if (this.ips.TryDequeue(out ip))
                {
                    ServicePointManager.ServerCertificateValidationCallback = (sender, certificate, chain, sslPolicyErrors) => true;
                    try
                    {

                        for (int i = 0; i < setting.From.Count; i++)
                        {
                            for (int j = 0; j < setting.To.Count; j++)
                            {
                                var dt = DateTime.Now;
                                var request = WebRequest.Create(
                                    string.Format(
                                        "https://{0}/otn/leftTicket/queryX?leftTicketDTO.train_date={1}&leftTicketDTO.from_station={2}&leftTicketDTO.to_station={3}&purpose_codes=ADULT",
                                        ip,
                                        setting.Date,
                                        setting.From[i],
                                        setting.To[j]
                                        )) as HttpWebRequest;
                                request.Host = "kyfw.12306.cn";
                                request.UserAgent = "Mozilla/5.0 (Linux; U; Android 2.3.6; zh-cn; GT-S5660 Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 MicroMessenger/4.5.255";
                                request.Timeout = 3000;
                                using (var response = request.GetResponse())
                                {
                                    var h = response.Headers[3];
                                    //Console.WriteLine(h);
                                    if ((DateTime.Now - dt).Milliseconds < 2000)
                                    {
                                        //ipStr = ipStr + "\r\n" + ip;
                                    }

                                    using (var sr = new StreamReader(response.GetResponseStream()))
                                    {
                                        var result = JsonConvert.DeserializeObject<Piao>(sr.ReadToEnd()).data.SingleOrDefault(p => setting.Code.Contains(p.queryLeftNewDTO.station_train_code) && p.queryLeftNewDTO.IsOptimal());
                                        Console.WriteLine("查询IP {0} {1}", ip, setting.Code.First());
                                        if (result != null && !string.IsNullOrEmpty(result.secretStr))
                                        {
                                            secretStr = result.secretStr;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ip + ":" + ex.Message);
                    }
                }
                else
                {
                    //IP里队列都查完了，退出
                    break;
                }
            }
        }
    }

    public class Setting
    {
        public Setting()
        {
            this.Ips = new List<string>();
            this.From = new List<string>();
            this.To = new List<string>();
            this.Code = new List<string>();
        }

        //所有cdnIP
        public List<string> Ips { get; set; }
        //出发站代码，如BJP,BXP,VNP都代表北京，可以去12306页面里去找这个数据
        public List<string> From { get; set; }
        //到站代码
        public List<string> To { get; set; }
        //车次，如Z5
        public List<string> Code { get; set; }
        //乘车日期
        public string Date { get; set; }
    }

    //映射/otn/leftTicket/queryX 返回的json对象
    public class Piao
    {
        public string validateMessagesShowId { get; set; }
        public bool status { get; set; }
        public List<PiaoData> data { get; set; }
    }

    public class PiaoData
    {
        public PiaoDTO queryLeftNewDTO { get; set; }
        public string secretStr { get; set; }
    }

    public class PiaoDTO
    {
        public string station_train_code { get; set; }
        //硬座
        public string yz_num { get; set; }
        //一等座
        public string zy_num { get; set; }
        //二等座
        public string ze_num { get; set; }
        //硬卧
        public string yw_num { get; set; }
        //软卧
        public string rw_num { get; set; }

        //是否有二等座或硬卧票
        public bool IsOptimal()
        {
            int num = 0;

            if (this.ze_num == "有")
                return true;

            if (int.TryParse(this.ze_num, out num))
            {
                if (num > 0)
                    return true;
            }

            num = 0;

            if (this.yw_num == "有")
                return true;

            if (int.TryParse(this.yw_num, out num))
            {
                if (num > 0)
                    return true;
            }

            return false;
        }
    }
}